/* Main */

// main.c
extern unsigned int __attribute__((aligned(16))) gulist[262144];
extern void *drawBuffer;
extern int exitRequest;
int running();
struct Vertex3DCNP {
	unsigned long color;
	float nx,ny,nz;
	float x,y,z;
};
struct Vertex3DTNP {
	float u,v;
	float nx,ny,nz;
	float x,y,z;
};
struct Vertex3DCP {
	unsigned long color;
	float x,y,z;
};
struct Vertex3DTCP {
	float u,v;
	unsigned long color;
	float x,y,z;
};
struct Vertex3DTP {
	float u,v;
	float x,y,z;
};
struct Vertex3DTPfast {
	short u,v;
	short x,y,z;
};

enum Buttons {
	BT_UP,BT_DOWN,BT_LEFT,BT_RIGHT,
	BT_TRIANGLE,BT_CIRCLE,BT_SQUARE,BT_CROSS,
	BT_LTRIGGER,BT_RTRIGGER,
	BT_START,BT_SELECT,BT_HOLD,
};

// image.c
typedef unsigned int Color;
typedef struct Image
{
        int textureWidth;  // the real width of data, 2^n with n>=0
        int textureHeight;  // the real height of data, 2^n with n>=0
        int imageWidth;  // the image width
        int imageHeight;	// the image height
        int isSwizzled;	// Is the image swizzled?
        int vram;		// Is the image in vram or not?
        Color* data;
} Image;
Image *loadPng(const char *filename);
void freeImage(Image *image);
void resetVRam();
extern int swizzleToVRam;
void swizzleFast(Image *source);
void saveImagePng(const char* filename, Color* data, int width, int height, int lineSize, int saveAlpha);
void saveImageTarga(const char* filename, Color* data, int width, int height, int lineSize, int saveAlpha);
enum FontId {
	FONT_HEADLINE,FONT_BODY,FONT_BODYHIGHLIGHT,FONT_MESSAGE,FONT_SMALL,FONT_SMALLHIGHLIGHT
};
void initFastFont();
void extentMessage(int *w,int *h, enum FontId fontId, const char *message);
void drawMessage(int x,int y, enum FontId fontId, const char *message);
void drawMessageAlpha(int x,int y, enum FontId fontId, const char *message,int alpha);
void drawMessageFormat(int x,int y,enum FontId fontId, const char *message);
Image *loadCell(const char *fname);
void drawCell(int x,int y,const char *fname);
void freeCells();
Image *newImage(int width,int height);

// sound.c
enum Sfx {
	SFX_MENUNEXT,SFX_MENUSELECT,SFX_TARGET,SFX_CHAOS,SFX_LOCKON,SFX_EXPLODE,SFX_POWERUP,SFX_WIN,SFX_LOSE,MAX_SFX
};
void initSound();
void updateSound();
void playSfx(enum Sfx id);
void playSound(int id);
void playWav(const char *fname);
void freeWavs();
void songVolume(int vol);

// gutools.c
void drawSprite(int sx, int sy, int width, int height, Image* source, int dx, int dy);
void drawSpriteAlpha(int sx, int sy, int width, int height, Image* source, int dx, int dy,int alpha);
void drawFilledRect(int x, int y, int width, int height, unsigned color);

struct AnimFrame {
	float scale[3];
	float translate[3];
	struct Vertex3DTPfast *vert;
};
// resonate.c
struct AnimMesh {
	Image *texture;
	int frameCount;
	int polyCount;
	int skinWidth,skinHeight;
	struct AnimFrame *frames;
};
struct AnimMotion {
	int startFrame;
	int loopFrame;
	int loopLength;
	int fps;
};
enum MotionType { MT_INACTIVE,MT_IDLE,MT_HIT,MT_EXPLODE,MT_FIRE,MT_CHAOS };
struct AnimModel {
	char name[64];
	struct AnimMesh *mesh[2];
	int meshCount;
	struct AnimMotion motion[16];
	int cullBackface;
	unsigned long explosionColor[2];
	float explosionRadius;
	float explosionYScale;
	int explosionDuration;	// in 1/1000ths of a second.
};
void newGame();
void update(unsigned long elapsed);
int handleJoy(enum Buttons button,int up);
int handleAnalog(int lx,int ly);
void draw();

// julia.c
void generateVisualization(float a,float b,Image *image,int id);
void generateJulia(float a,float b,Image *image);
void generateWallpaper(float a,float b,Image *image);
void generatePlasma(float a,float b,Image *image);
void copyPalette(unsigned int *palette);

// md2fast.h
struct AnimMesh *md2Load(const char *fname);
void md2GetFractionalFrame(struct AnimMesh *mesh,int frame,int nextFrame,float fraction,struct Vertex3DTP *buffer,float *minMax);
void md2Free(struct AnimMesh *mesh);

// resonate.c
enum GameMode { MENUMODE,PAUSEMODE,SUMMARYMODE,USERLEVELMODE,OPTIONSMODE,ONLINEMODE,HIGHSCORESMODE,ONLINEHIGHSCORESMODE,INSTRUCTIONSMODE,CREDITSMODE,GAMEMODE, } ;
extern enum GameMode gameMode;
extern enum GameMode newMode;
extern int gameModeTimer;
extern int enemyHit;
extern int enemyMiss;
extern int gameScore;
extern int supportHit;
extern int supportMiss;
int handleJoyGame(enum Buttons button,int up);
const char *getCheckSum();
const char *getName();

// menu.c
extern struct Options {
	int invert;		// invert y: 1=on 0=off
	float speed;	// defaults to 2.4f
	int music;		// volume 0 to 10
	int sfx;		// 1=on 0=off
} options;
void setMode(enum GameMode mode);
extern int menuTimer;
extern int menuItem;
extern char alertMessage[256];
extern char userLevelList[256][64];
extern int userLevelCount;
extern int userLevelFirst;
extern int menuMax;
void drawMenu();
void loadOptions();

// website.c
void httpInit();
void httpTerm();
int post(const char *url,const char *body,int bodyLength,char *response,int *responseLength);
int fetch(const char *url,char *response,int *responseLength);
